;*********************************************************************
;                                                                    *
;           FILENAME:   DEMOAPP.ASM                                  *
;                                                                    *
;*********************************************************************
    .org 8000h

;*********************************************************************
;                                                                    *
; CARD HEADER : This must be the first thing on the card
;                                                                    *
;*********************************************************************

    ; Card header string
    .text AMS1
    .dw 00101h ; Card version number.
    .dw 01234h ; Unique ID.

    .dw App_Start ; Code start address

; A few definitions:

; The display window parameters

WINDOW_LEFT .equ 15 ; Left Side
WINDOW_RIGHT .equ 220 ; Right Side
WINDOW_TOP .equ 130 ; Top
WINDOW_BOTTOM .equ 180 ; Bottom

; And some fixed store locations for pointers, etc.

mem_ptr .equ 0fffeh ; memory pointer
win_ptr .equ 0fffch ; Window buffer pointer
X_PEN .equ 0fffah ; Pen X coordinate store
Y_PEN .equ 0fff8h ; Pen Y coordinate store

;*********************************************************************
; This is where the code starts

App_Start:

; Start by saying hello to the world in the form of a beep.

    ld iy, 013ah ; Load the BLIST vector fo 'Beep'
    call 0003h ; Call 'Beep' via SystemCall

; In order to handle the fixed output strings most efficiently, we
; need to copy them into main RAM, so first reserve some
; memory for them on the heap.

; Memory required = length of fixed strings,
;                   + maximum input string length
;                   + 1 for the null terminator on the
;                                         input string.

    ld hl, StringEnd - NameStr + 22 ; Get some memory
    ld iy, 070h ; Load BLIST vector for 'malloc'
    call 0003h ; Call 'malloc' via SystemCall
    ld (mem_ptr), hl ; Save the pointer to the memory
                     ; in a fixed pointer store

; On return from malloc, check that memory has been allocated.

    ld a, l ; Check for null => no memory
    or h
    jp z, exit_app_1 ; Exit if no memory

; malloc was successful, so now copy the strings into it.

; First copy some string data into the heap

    ld bc, StringEnd - NameStr ; Block size to read
    push bc
    ld de, NameStr - 8000h ; Card address lower word
    ld bc, 0h ; Card address upper word

; HL already has destination address

    ld iy, 0124h ; Load the BLIST vector for CardRead
    call 0003h
    pop af ; Remove parameters from stack

    ld a, c ; Test for success
    or b ; Zero flag set if Card Read failed.

    jp z, exit_app_2 ; Read failed, so exit the
                     ; application
                     ; But free the malloc'd space first

; String data have now been read into the heap, so now call up
; the recogniser.

; Set up the parameters for the RecogniserInput call.

    ld hl, 0 ; No user button
    push hl
    ld bc, 0 ; Allow blank entries
    push bc
    ld bc, 255 ; Recognise all characters
    push bc
    ld bc, 310 ; Bottom of recogniser box
    push bc
    ld bc, 3 ; Three lines of input,
             ; (= max 21 characters)
    ld hl, (mem_ptr) ; Point to input Buffer
    ld de, StringEnd - NameStr ; Offset to input string
                               ; buffer
    add hl, de ; Get pointer to input
               ; string buffer
    ex de, hl  ; Put in DE

    ld hl, (mem_ptr) ; Point to Title String

    ld iy, 00e2h ; BLIST vector for RecogniserInput
    call 0003h

    pop af ; Remove parameters from stack
    pop af
    pop af
    pop af

    ld a, c ; Test return value for TRUE/FALSE
    or a
    jr nz, keep_going ; Continue if user accepted input,
                      ; (return = TRUE)
    jp exit_app_2     ; Else abort the application
                      ; freeing the malloc'd space first

; We now have the input string from the user, so lets display
; it in a window

keep_going:

    ld bc, WINDOW_BOTTOM ; Set y2
    push bc
    ld bc, WINDOW_RIGHT ; Set x2
    ld de, WINDOW_TOP ; Set y1
    ld hl, WINDOW_LEFT ; Set x1

    ld iy, 00d2h ; BLIST vector for CreateWindow
    call 0003h

    pop af ; Adjust the stack
    ld (win_ptr), hl ; Store returned pointer

; There us now a window displayed on the screen.
; Whatever was on the screen before under the windows is
; saved in the buffer pointed to by 'win_ptr'

; Now write 'Hello' centred across the window

    ld hl, (mem_ptr) ; Point to "Hello" string
    ld bc, HelloStr - NameStr ; Offset to 'Hello' in strings
    add hl, bc
    push hl
    pop bc
    ld de, WINDOW_TOP + 10 ; Y coordinate for output
    ld HL, WINDOW_LEFT + 82 ; X coordinate for output
                            ; (centre justify)

    ld iy, 00a6h ; BLIST vector for outtextxy
    call 0003h

; Now lets find out the length of the string input

    ld hl, (mem_ptr) ; Point to start of strings
    ld bc, StringEnd - NameStr ; Length of fixed strings
    add hl, bc

    ld iy, 0104h ; Load the BLIST vector
    call 0003h

; Length of string is now in BC. Can assume value in B is zero
; as maximum input length is 21 characters.

; Now do the centre justification

; Basic formula required is :
;
;   Start X coordinate of string = Left coordinate of window
;                                  + Half width of window
;                                  - Half width of string in pixels
;
; Width of string in pixels = characters * 8, (for either TIMES font)

    xor a ; Clear carry
    ld a, c ; Get char count
    rla ; *4 = (char count * char width / 2)
    rla ; gives half pixel width

    ld c, a ; Save in C

    ld A, WINDOW_RIGHT - WINDOW_LEFT ; Get window width
    rra ; Divide by 2
    add a, WINDOW_LEFT ; add coordinate of left side of window
    sub c ; Finally subtract half width of string

; The accumulator now has the start X coordinate at which to
; output the string

    ld hl, (mem_ptr) ; Point to start of strings
    ld bc, StringEnd - NameStr ; Length of fixed strings
    add hl, bc ; Point to input buffer
    ld b, h ; Move into BC
    ld c, l

    ld de, WINDOW_TOP + 30 ; Y coordinate for output
    ld l, a ; Calculated X coordinate
    ld h, 0 ; from above

    ld iy, 00a6h ; BLIST vector for outtextxy
    call 0003h

; Display is now complete, so wait for the pen to be put down
; in the window

; First flush the pen to ensure that there are no old pen data
; in the queue.

    ld iy, 0118h ; BLIST vector for 'PenFlush'
    call 0003h

not_in_window:

; PenRead requires teh calling routine to have two integer store
; locations into which it can write the pen coordinates. The
; addresses of these words are passed as parameters to PenRead.

    ld hl, 0 ; Initialise the two words
    ld (X_PEN), hl ; Zero the X pen value
    ld (Y_PEN), hl ; Zero the Y pen value

; In case teh pen is still seen as down from the exit from the
; recogniser, wait to see the pen up first.

WaitPenUp:

    ld hl, Y_PEN ; Get address of Y coordinate store
    ex de, hl

    ld hl, X_PEN ; Get address of X coordinate store

    ld iy, 011ch ; Load the BLIST vector for PenRead
    call 0003h

    ld a, c ; Test the returned status
    cp 2 ; Is it down?

    jr nz, WaitPenUp ; Loop if pen is not up

; Pen us now up, (coordinates irrelevant).
; Now wait for the pen to be put down again

WaitPenDown:

    ld hl, Y_PEN ; Get address of Y coordinate store
    ex de, hl

    ld hl, X_PEN ; Get address of X coordinate store

    ld iy, 011ch ; Load the BLIST vector for PenRead
    call 0003h

    ld a, c ; Test the returned status
    cp 1 ; Is it up?

    jr nz, WaitPenDown ; Loop if pen is not down

; Pen is now down, at coordinates as recorded in the two store
; locations on the stack

; See if the pen has been put down in the window.

; First test the X coordinate bounds

    ld hl, (X_PEN) ; Get X coordinate of pen

    xor a ; clear the carry
    ld bc, WINDOW_LEFT ; Left side of window
    sbc hl, bc ; Test for left bound
    jr c, not_in_window ; Look again if not in the window

    ld bc, WINDOW_RIGHT - WINDOW_LEFT ; Right bound - left bound
    sbc hl, bc ; Test for right bound
    jr nc, not_in_window ; Look again if not in the window

; Now test the Y coordinate bounds

    ld hl, (Y_PEN) ; Get Y coordinate of pen

    xor a ; clear the carry

    ld bc, WINDOW_TOP ; Top side of window
    sbc hl, bc ; Test for top bound
    jr c, not_in_window ; Look again if not in the window.

    ld bc, WINDOW_BOTTOM - WINDOW_TOP ; Bottom bound - top bound
    sbc hl, bc ; Test for bottom bound
    jr nc, not_in_window ; Look again if not in the window.

; Pen has been put down in the window, so...

    ; Now close the window

    ld hl, (win_ptr) ; Get pointer to saved image
    ld iy, 00d4h ; BLIST vector to CloseWindow
    call 0003h

    ; Free the space on the heap

exit_app_2:
    ld hl, (mem_ptr) ; Get pointer to memory block.
    ld iy, 072h ; BLIST vector for free
    call 0003h

exit_app_1:
    ret ; Return to main system software

;*********************************************************************
;
; Constant data for standard output strings
;
;*********************************************************************

NameStr:
    .text Please Write Your name
    .db 0

HelloStr:
    .text Hello
    .db 0

StringEnd:
    .db 0

;*********************************************************************

    .end

;*********************************************************************
